@@ -19,6 +19,8 @@ import configparser
 import os
 import sys
 import gettext
+import packaging.version as version
+import pikepdf
 from gi.repository import Gdk
 from gi.repository import Gtk
 
@@ -115,8 +117,13 @@ class Config(object):
         self.data.read(Config._config_file(domain))
         if 'preferences' not in self.data:
             self.data.add_section('preferences')
+        preferences = self.data['preferences']
+        if 'save-window-geometry' not in preferences:
+            preferences['save-window-geometry'] = 'true'
         if 'print-settings' not in self.data:
             self.data.add_section('print-settings')
+        if 'image-export' not in self.data:
+            self.data.add_section('image-export')
         if 'accelerators' not in self.data:
             self.data.add_section('accelerators')
         a = self.data['accelerators']
@@ -129,6 +136,7 @@ class Config(object):
         self.popup_menu_accels = [
             Gtk.accelerator_parse(x) for x in a["context-menu"].split()
         ]
+        self.has_pikepdf8 = version.parse(pikepdf.__version__) >= version.Version("8.0")
 
     def is_popup_key_event(self, keyevent):
         for key, mods in self.popup_menu_accels:
@@ -137,6 +145,9 @@ class Config(object):
                 return True
         return False
 
+    def save_window_geometry(self):
+        return self.data.getboolean('preferences', 'save-window-geometry', fallback=True)
+
     def window_size(self):
         ds = Gdk.Screen.get_default()
         return self.data.getint('window', 'width', fallback=int(min(700, ds.get_width() / 2))), \
@@ -146,13 +157,6 @@ class Config(object):
         self.data.set('window', 'width', str(size[0]))
         self.data.set('window', 'height', str(size[1]))
 
-    def set_position(self, position):
-        self.data.set('window', 'root_x', str(position[0]))
-        self.data.set('window', 'root_y', str(position[1]))
-
-    def position(self):
-        return self.data.getint('window', 'root_x', fallback=10), self.data.getint('window', 'root_y', fallback=10)
-
     def maximized(self):
         return self.data.getboolean('window', 'maximized', fallback=False)
 
@@ -189,6 +193,15 @@ class Config(object):
     def set_theme(self, theme):
         self.data.set('preferences', 'theme', theme)
 
+    def start_with_empty(self):
+        return (self.data.getboolean('preferences', 'start-with-empty', fallback=False)
+                if self.has_pikepdf8 else True)
+
+    def set_start_with_empty(self, start_with_empty):
+        if not self.has_pikepdf8:
+            start_with_empty = True
+        self.data.set('preferences', 'start-with-empty', str(start_with_empty))
+
     def scale_mode(self):
         return self.data.get('print-settings', 'scale-mode', fallback="PRINTABLE")
 
@@ -201,6 +214,24 @@ class Config(object):
     def set_auto_rotate(self, enabled):
         self.data.set('print-settings', 'auto-rotate', str(enabled))
 
+    def image_ppi(self):
+        return self.data.getint('image-export', 'image-ppi', fallback=150)
+
+    def set_image_ppi(self, ppi):
+        self.data.set('image-export', 'image-ppi', str(ppi))
+
+    def optimize(self):
+        return self.data.getboolean('image-export', 'optimize', fallback=False)
+
+    def set_optimize(self, optimize):
+        self.data.set('image-export', 'optimize', str(optimize))
+
+    def greyscale(self):
+        return self.data.getboolean('image-export', 'greyscale', fallback=False)
+
+    def set_greyscale(self, greyscale):
+        self.data.set('image-export', 'greyscale', str(greyscale))
+
     def save(self):
         conffile = Config._config_file(self.domain)
         os.makedirs(os.path.dirname(conffile), exist_ok=True)
@@ -268,11 +299,38 @@ class Config(object):
         psettings = PrintSettingsWidget(self.scale_mode(), self.auto_rotate())
         frame3.add(psettings)
         d.vbox.pack_start(frame3, False, False, 8)
+        if self.has_pikepdf8:
+            frame4 = Gtk.Frame(label=_("Saving/exporting to single file"), margin=8)
+            cb_retain = Gtk.CheckButton(
+                label=_("Retain document-level information of the first opened file"),
+                margin=8)
+            cb_retain.set_active(not self.start_with_empty())
+            frame4.add(cb_retain)
+            d.vbox.pack_start(frame4, False, False, 8)
+        else:
+            # prevent CodeQL false positive "uninitialized local variable"
+            cb_retain = None
+        frame5 = Gtk.Frame(label=_("Image Export"), margin=8)
+        grid5 = Gtk.Grid(row_spacing=6, column_spacing=12, border_width=12)
+        label5 = Gtk.Label(_("Pixels/inch:"))
+        grid5.attach(label5, 0, 1, 1, 1)
+        sb_image_ppi = Gtk.SpinButton.new_with_range(1, 1200, 1)
+        sb_image_ppi.props.width_chars = 8
+        sb_image_ppi.set_value(self.image_ppi())
+        grid5.attach(sb_image_ppi, 1, 1, 1, 1)
+        cb_optimize = Gtk.CheckButton(label=_("Optimize"), margin=8)
+        cb_optimize.set_active(self.optimize())
+        grid5.attach(cb_optimize, 0, 2, 1, 1)
+        cb_greyscale = Gtk.CheckButton(label=_("Greyscale"), margin=8)
+        cb_greyscale.set_active(self.greyscale())
+        grid5.attach(cb_greyscale, 1, 2, 1, 1)
+        frame5.add(grid5)
+        d.vbox.pack_start(frame5, False, False, 8)
         t = _("For more options see:")
-        frame4 = Gtk.Frame(label=t, shadow_type=Gtk.ShadowType.NONE, margin=8)
-        label4 = Gtk.Label(self._config_file(self.domain), selectable=True, margin=8)
-        frame4.add(label4)
-        d.vbox.pack_start(frame4, False, False, 8)
+        frame6 = Gtk.Frame(label=t, shadow_type=Gtk.ShadowType.NONE, margin=8)
+        label6 = Gtk.Label(self._config_file(self.domain), selectable=True, margin=8)
+        frame6.add(label6)
+        d.vbox.pack_start(frame6, False, False, 8)
 
         langs = []
         if os.path.isdir(localedir):
@@ -306,6 +364,11 @@ class Config(object):
             num2 = combo2.get_active()
             theme = themes[num2] if num2 != 0 else ""
             self.set_theme(theme)
+            if self.has_pikepdf8:
+                self.set_start_with_empty(not cb_retain.get_active())
             self.set_scale_mode(psettings.get_scale_mode())
             self.set_auto_rotate(psettings.get_auto_rotate())
+            self.set_image_ppi(sb_image_ppi.get_value_as_int())
+            self.set_optimize(cb_optimize.get_active())
+            self.set_greyscale(cb_greyscale.get_active())
         d.destroy()
